#######################################################################
#                                                                     #
# Magnitude and direction of parasite-induced phenotypic alterations: #
#               a meta-analysis in acanthocephalans                   #
#                     R CODES (all analyses)                          #
#                       (Marion Fayard, PhD)                          #
#                                                                     #
#######################################################################

library(MCMCglmm)
library(MuMIn)
library(metafor)
library(phytools)


################
# 1) PHYLOGENY #
################
### importing phylogenetic tree (complete)
phylo=read.tree("interro_brute_ncbi_nettoyee3_muscle_mafft.newick")
summary.phylo(phylo) # checks if tree has imported correctly (number of tips)
phylo$node.label <- NULL
phylo2=drop.tip(phylo, tip=c(2:3)) # to keep 1 rotifera (to root) and other acantho
is.rooted(phylo2) # checks if the tree is rooted (necessary)
phylo3=force.ultrametric(phylo2, method="extend") # to make the tree ultrametric
is.ultrametric(phylo3) # checks if the tree is ultrametric (necessary)
plot(phylo3)


####################
# 2) META-ANALYSIS #
####################
options(max.print=100000)
a=read.table("analysis4_3STANDCONS(seq)OK.csv", h=T) # COMPLETE data
a

a=read.table("analysis4_3STANDCONS(seq)OKambig.csv", h=T) # data without AMBIGUOUS siging 
a

a=read.table("analysis4_3STANDCONS(seq)_REDUCED_OK.txt", h=T) # for SENSITIVITY analysis -> subset
a

a=a[a$parasite_maturity=="cystacanth",] # for analyses with CYSTACANTHS only
a

z1 <- 0.5*log((1+a$r1)/(1-a$r1)) # r to Fisher's z transformation
z1
var1=1/(a$n1-3) # gives variance for each effect size estimate
var1
qqnorm(z1)
qqline(z1) # checks normality, if ok, then gather z1, var and data a:
b=data.frame(var1,z1,a)


#####################################################################
#### ABSOLUTE META ANALYTIC MEAN
#--------------------------------
# function for applying folded normal distribution to a MCMCglmm posterior distribution (from Morrissey, 2016a,b)
mu.fnorm <-  function(mu, sigma){
  dnorm(mu, 0, sigma)*2*sigma^2 + mu*(2*pnorm(mu, 0, sigma) -1)
}
# function to calculate the confidence interval
var.fnorm <- function(mu, sigma){
  mu^2 + sigma^2 - (sigma*sqrt(2/pi)*exp((-1*mu^2)/(2*sigma^2)) + mu*(1-2*pnorm(-1*mu/sigma, 0, 1)))^2
}
# folded function from Noble et al.(2017) to get absolute mean and CI
folded <- function(mu, sigma){
  mu.fnorm.mode <- c()
  CI.mu <- matrix(nrow = length(colnames(mu)), ncol = 2)
  
  for(i in 1:length(colnames(mu))){
    mu.fnorm <-  mu.fnorm(mu[,i], sigma)
    mu.fnorm.mode[i] <- as.numeric(MCMCglmm::posterior.mode(as.mcmc(mu.fnorm)))
    CI.mu[i,]       <- as.numeric(coda::HPDinterval(as.mcmc(mu.fnorm))) 
  }
  colnames(CI.mu) <- c("lower", "upper")
  var.fnorm.mode <- c()
  
  for(i in 1:length(colnames(mu))){
    var.fnorm <- var.fnorm(mu[,i], sigma)
    var.fnorm.mode[i] <- as.numeric(MCMCglmm::posterior.mode(as.mcmc(var.fnorm)))
  }
  return(data.frame(coef = colnames(mu), mu = mu.fnorm.mode, var = var.fnorm.mode, CI = CI.mu))
}

### meta-analytic models
### non-informative priors
prior=list(R=list(V=1,nu=0.002), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002)))
mod1=MCMCglmm(z1~1, mev=var1, random=~study+parasite_species,prior=prior,nitt=500000,thin=250,burnin=50000, data=b)
summary(mod1)
# estimate the posterior distribution for folded normal
postVCV<- mod1$VCV
postSol<- mod1$Sol
est.fnorm <- folded(postSol, sqrt(rowSums(postVCV[,c(1:2,4)])))
est.fnorm # absolute Zr meta-analytic mean and confidence interval
# back-transformation to get r (post mean and 95% CI)
rmeta=(exp(2*est.fnorm[2])-1)/(exp(2*est.fnorm[2])+1) # back-transformation from z 
rmeta
rinf=(exp(2*est.fnorm[4])-1)/(exp(2*est.fnorm[4])+1) # back-transformation from z 
rinf
rsup=(exp(2*est.fnorm[5])-1)/(exp(2*est.fnorm[5])+1) # back-transformation from z 
rsup

### non-informative priors and phylogenetic matrix
b$animal <- as.factor(b$parasite_species) # Pedigree function needs column called "animal"
# WARNING -> name of the matrice should represent the column in data associated with random term
# and all levels of random term should appear as row names in the matrice
prior=list(R=list(V=1,nu=0.002), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod5=MCMCglmm(z1~1, mev=var1, random=~study+parasite_species+animal,prior=prior,pedigree=phylo3,nitt=500000,thin=250,burnin=50000, data=b)
summary(mod5)
# estimate the posterior distribution for folded normal
postVCV<- mod5$VCV
head(postVCV)
postSol<- mod5$Sol
head(postSol)
est.fnorm <- folded(postSol, sqrt(rowSums(postVCV[,c(1:3,5)])))
est.fnorm # absolute Zr meta-analytic mean and confidence interval
# back-transformation to get r (post mean and 95% CI)
rmeta=(exp(2*est.fnorm[2])-1)/(exp(2*est.fnorm[2])+1) # back-transformation from z 
rmeta
rinf=(exp(2*est.fnorm[4])-1)/(exp(2*est.fnorm[4])+1) # back-transformation from z 
rinf
rsup=(exp(2*est.fnorm[5])-1)/(exp(2*est.fnorm[5])+1) # back-transformation from z 
rsup

### absolute means for factors levels 
## trait categories
prior=list(R=list(R1=list(V=diag(5),nu=0.002),R2=list(V=1,nu=0.002)),G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod9=MCMCglmm(z1~category,random=~study+parasite_species+animal,mev=var1,rcov=~idh(category):units,prior=prior,pedigree=phylo3,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod9)
head(mod9$Sol)
head(mod9$VCV)
# posterior mean distributions for each trait category
behaviour <- mod9$Sol[,1]
lht <- mod9$Sol[,1] + mod9$Sol[,2]
morpho <- mod9$Sol[,1] + mod9$Sol[,3]
physio <- mod9$Sol[,1] + mod9$Sol[,4]
predation <- mod9$Sol[,1] + mod9$Sol[,5]
category_est <- posterior.mode(cbind(behaviour, lht, morpho, physio, predation))
category_CI <- HPDinterval(as.mcmc(cbind(behaviour, lht, morpho, physio, predation)))
category <- round(data.frame(n = 1:5, category_est, category_CI), digits = 2)
category
# estimate the posterior distribution for folded normal
f_behaviour <- mu.fnorm(mu = mod9$Sol[,1], sigma = rowSums(mod9$VCV[,c(1:3,5)]))
f_lht <- mu.fnorm(mu = mod9$Sol[,1] + mod9$Sol[,2], sigma = rowSums(mod9$VCV[,c(1:3,6)]))
f_morpho <- mu.fnorm(mu = mod9$Sol[,1] + mod9$Sol[,3], sigma = rowSums(mod9$VCV[,c(1:3,7)]))
f_physio <- mu.fnorm(mod9$Sol[,1] + mod9$Sol[,4], sigma = rowSums(mod9$VCV[,c(1:3,8)]))
f_predation <- mu.fnorm(mod9$Sol[,1] + mod9$Sol[,5], sigma = rowSums(mod9$VCV[,c(1:3,9)]))
f_category_est <- posterior.mode(cbind(f_behaviour, f_lht, f_morpho, f_physio, f_predation))
f_category_CI <- HPDinterval(as.mcmc(cbind(f_behaviour, f_lht, f_morpho, f_physio, f_predation)))
f_category <- round(data.frame(n = 1:5, f_category_est, f_category_CI), digits = 2)
f_category # absolute means for each trait category
# back-transformation to get r (post mean and 95% CI)
rmeta_behaviour=(exp(2*f_category[1,2])-1)/(exp(2*f_category[1,2])+1) # back-transformation from z 
rmeta_behaviour
rinf_behaviour=(exp(2*f_category[1,3])-1)/(exp(2*f_category[1,3])+1) # back-transformation from z 
rinf_behaviour
rsup_behaviour=(exp(2*f_category[1,4])-1)/(exp(2*f_category[1,4])+1) # back-transformation from z 
rsup_behaviour
rmeta_lht=(exp(2*f_category[2,2])-1)/(exp(2*f_category[2,2])+1) # back-transformation from z 
rmeta_lht
rinf_lht=(exp(2*f_category[2,3])-1)/(exp(2*f_category[2,3])+1) # back-transformation from z 
rinf_lht
rsup_lht=(exp(2*f_category[2,4])-1)/(exp(2*f_category[2,4])+1) # back-transformation from z 
rsup_lht
rmeta_morpho=(exp(2*f_category[3,2])-1)/(exp(2*f_category[3,2])+1) # back-transformation from z 
rmeta_morpho
rinf_morpho=(exp(2*f_category[3,3])-1)/(exp(2*f_category[3,3])+1) # back-transformation from z 
rinf_morpho
rsup_morpho=(exp(2*f_category[3,4])-1)/(exp(2*f_category[3,4])+1) # back-transformation from z 
rsup_morpho
rmeta_physio=(exp(2*f_category[4,2])-1)/(exp(2*f_category[4,2])+1) # back-transformation from z 
rmeta_physio
rinf_physio=(exp(2*f_category[4,3])-1)/(exp(2*f_category[4,3])+1) # back-transformation from z 
rinf_physio
rsup_physio=(exp(2*f_category[4,4])-1)/(exp(2*f_category[4,4])+1) # back-transformation from z 
rsup_physio
rmeta_predation=(exp(2*f_category[5,2])-1)/(exp(2*f_category[5,2])+1) # back-transformation from z 
rmeta_predation
rinf_predation=(exp(2*f_category[5,3])-1)/(exp(2*f_category[5,3])+1) # back-transformation from z 
rinf_predation
rsup_predation=(exp(2*f_category[5,4])-1)/(exp(2*f_category[5,4])+1) # back-transformation from z 
rsup_predation

## parasite maturity
prior=list(R=list(R1=list(V=diag(2),nu=0.002),R2=list(V=1,nu=0.002)),G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod10=MCMCglmm(z1~parasite_maturity,random=~study+parasite_species+animal,mev=var1,rcov=~idh(parasite_maturity):units,prior=prior,pedigree=phylo3,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod10)
head(mod10$Sol)
head(mod10$VCV)
# posterior mean distributions for each parasite level
acanthella <- mod10$Sol[,1]
cystacanth <- mod10$Sol[,1] + mod10$Sol[,2]
parasite_est <- posterior.mode(cbind(acanthella, cystacanth))
parasite_CI <- HPDinterval(as.mcmc(cbind(acanthella, cystacanth)))
parasite <- round(data.frame(n = 1:2, parasite_est, parasite_CI), digits = 2)
parasite
# estimate the posterior distribution for folded normal
f_acanthella <- mu.fnorm(mu = mod10$Sol[,1], sigma = rowSums(mod10$VCV[,c(1:3,5)]))
f_cystacanth <- mu.fnorm(mu = mod10$Sol[,1] + mod10$Sol[,2], sigma = rowSums(mod10$VCV[,c(1:3,6)]))
f_parasite_est <- posterior.mode(cbind(f_acanthella, f_cystacanth))
f_parasite_CI <- HPDinterval(as.mcmc(cbind(f_acanthella, f_cystacanth)))
f_parasite <- round(data.frame(n = 1:2, f_parasite_est, f_parasite_CI), digits = 2)
f_parasite # absolute means for each parasite level
# back-transformation to get r (post mean and 95% CI)
rmeta_acanthella=(exp(2*f_parasite[1,2])-1)/(exp(2*f_parasite[1,2])+1) # back-transformation from z 
rmeta_acanthella
rinf_acanthella=(exp(2*f_parasite[1,3])-1)/(exp(2*f_parasite[1,3])+1) # back-transformation from z 
rinf_acanthella
rsup_acanthella=(exp(2*f_parasite[1,4])-1)/(exp(2*f_parasite[1,4])+1) # back-transformation from z 
rsup_acanthella
rmeta_cystacanth=(exp(2*f_parasite[2,2])-1)/(exp(2*f_parasite[2,2])+1) # back-transformation from z 
rmeta_cystacanth
rinf_cystacanth=(exp(2*f_parasite[2,3])-1)/(exp(2*f_parasite[2,3])+1) # back-transformation from z 
rinf_cystacanth
rsup_cystacanth=(exp(2*f_parasite[2,4])-1)/(exp(2*f_parasite[2,4])+1) # back-transformation from z 
rsup_cystacanth

## infection type
prior=list(R=list(R1=list(V=diag(2),nu=0.002),R2=list(V=1,nu=0.002)),G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod11=MCMCglmm(z1~infection_type,random=~study+parasite_species+animal,mev=var1,rcov=~idh(infection_type):units,prior=prior,pedigree=phylo3,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod11)
head(mod11$Sol)
head(mod11$VCV)
# posterior mean distributions for each infection level
expe <- mod11$Sol[,1]
natural <- mod11$Sol[,1] + mod11$Sol[,2]
infection_est <- posterior.mode(cbind(expe, natural))
infection_CI <- HPDinterval(as.mcmc(cbind(expe, natural)))
infection <- round(data.frame(n = 1:2, infection_est, infection_CI), digits = 2)
infection
# estimate the posterior distribution for folded normal
f_expe <- mu.fnorm(mu = mod11$Sol[,1], sigma = rowSums(mod11$VCV[,c(1:3,5)]))
f_natural <- mu.fnorm(mu = mod11$Sol[,1] + mod11$Sol[,2], sigma = rowSums(mod11$VCV[,c(1:3,6)]))
f_infection_est <- posterior.mode(cbind(f_expe, f_natural))
f_infection_CI <- HPDinterval(as.mcmc(cbind(f_expe, f_natural)))
f_infection <- round(data.frame(n = 1:2, f_infection_est, f_infection_CI), digits = 2)
f_infection # absolute means for each infection level
# back-transformation to get r (post mean and 95% CI)
rmeta_expe=(exp(2*f_infection[1,2])-1)/(exp(2*f_infection[1,2])+1) # back-transformation from z 
rmeta_expe
rinf_expe=(exp(2*f_infection[1,3])-1)/(exp(2*f_infection[1,3])+1) # back-transformation from z
rinf_expe
rsup_expe=(exp(2*f_infection[1,4])-1)/(exp(2*f_infection[1,4])+1) # back-transformation from z 
rsup_expe
rmeta_natural=(exp(2*f_infection[2,2])-1)/(exp(2*f_infection[2,2])+1) # back-transformation from z 
rmeta_natural
rinf_natural=(exp(2*f_infection[2,3])-1)/(exp(2*f_infection[2,3])+1) # back-transformation from z 
rinf_natural
rsup_natural=(exp(2*f_infection[2,4])-1)/(exp(2*f_infection[2,4])+1) # back-transformation from z 
rsup_natural


#####################################################
##### SIGNED META ANALYTIC MEAN
#-------------------------------
### non-informative priors and no phylogenetic matrix
prior=list(R=list(V=1,nu=0.002), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002)))
mod1=MCMCglmm(z1~1, mev=var1, random=~study+parasite_species,prior=prior,nitt=500000,thin=250,burnin=50000, data=b)
summary(mod1)
# back-transformation to get r (post mean and 95% CI)
rmeta=(exp(2*summary(mod1)$solutions[1])-1)/(exp(2*summary(mod1)$solutions[1])+1) # back-transformation from z 
rmeta
rinf=(exp(2*summary(mod1)$solutions[2])-1)/(exp(2*summary(mod1)$solutions[2])+1) # back-transformation from z 
rinf
rsup=(exp(2*summary(mod1)$solutions[3])-1)/(exp(2*summary(mod1)$solutions[3])+1) # back-transformation from z 
rsup
# checking auto-correlations between samplings (less than 0.10 is good from the second)
autocorr.diag(mod1$Sol) # for intercept
autocorr.diag(mod1$VCV) # for random effects
## creating the same model to check convergence between models
mod2=MCMCglmm(z1~1, mev=var1, random=~study+parasite_species,prior=prior,nitt=500000,thin=250,burnin=50000,start=list(QUASI=FALSE),data=b)
summary(mod2)
# diagnose convergence (closer to 1 = better)
gelman.diag(mcmc.list(mod1$Sol,mod2$Sol), multivariate=F) # for the intercept
gelman.diag(mcmc.list(mod1$VCV,mod2$VCV), multivariate=F) # for the random effects
gelman.plot(mcmc.list(mod1$Sol,mod2$Sol), auto.layout = F) # to see convergence

##############################################
### expanded priors and no phylogenetic matrix
prior=list(R=list(V=1,nu=0.002), G=list(G1=list(V=1,nu=1,alpha.mu=0,alpha.V =1000),G2=list(V=1,nu=1,alpha.mu=0,alpha.V =1000)))
mod3=MCMCglmm(z1~1, mev=var1, random=~study+parasite_species,prior=prior,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod3)
# back-transformation to get r (post mean and 95% CI)
rmeta=(exp(2*summary(mod3)$solutions[1])-1)/(exp(2*summary(mod3)$solutions[1])+1) # back-transformation from z 
rmeta
rinf=(exp(2*summary(mod3)$solutions[2])-1)/(exp(2*summary(mod3)$solutions[2])+1) # back-transformation from z 
rinf
rsup=(exp(2*summary(mod3)$solutions[3])-1)/(exp(2*summary(mod3)$solutions[3])+1) # back-transformation from z 
rsup
# checking auto-correlations between samplings (less than 0.10 is good)
autocorr.diag(mod3$Sol) # for intercept
autocorr.diag(mod3$VCV) # for random effects
## creating the same model to check convergence between models
mod4=MCMCglmm(z1~1, mev=var1, random=~study+parasite_species,prior=prior,nitt=500000,thin=250,burnin=50000,start=list(QUASI=FALSE),data=b)
summary(mod4)
# diagnose convergence (closer to 1 = better)
gelman.diag(mcmc.list(mod3$Sol,mod4$Sol), multivariate=F) # for the intercept
gelman.diag(mcmc.list(mod3$VCV,mod4$VCV), multivariate=F) # for the random effects
gelman.plot(mcmc.list(mod3$Sol,mod4$Sol), auto.layout = F) # to see convergence
# checking convergence between models with different priors (non-informative and expanded)
gelman.diag(mcmc.list(mod1$Sol,mod3$Sol), multivariate=F) # for the intercept
gelman.diag(mcmc.list(mod1$VCV,mod3$VCV), multivariate=F) # for the random effects
gelman.plot(mcmc.list(mod1$Sol,mod3$Sol), auto.layout = F) # to see convergence

#####################################################
### phylogenetic meta-analysis
b$animal <- as.factor(b$parasite_species) # Pedigree function needs column called "animal"
##################################################
### non-informative priors and phylogenetic matrix
# WARNING -> name of the matrice should represent the column in data associated with random term
# and all levels of random term should appear as row names in the matrice
prior=list(R=list(V=1,nu=0.002), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod5=MCMCglmm(z1~1, mev=var1, random=~study+parasite_species+animal,prior=prior,pedigree=phylo3,nitt=500000,thin=250,burnin=50000, data=b)
summary(mod5)
#back-transformation to get r (post mean and 95% CI)
rmeta=(exp(2*summary(mod5)$solutions[1])-1)/(exp(2*summary(mod5)$solutions[1])+1) # back-transformation from z 
rmeta
rinf=(exp(2*summary(mod5)$solutions[2])-1)/(exp(2*summary(mod5)$solutions[2])+1) # back-transformation from z 
rinf
rsup=(exp(2*summary(mod5)$solutions[3])-1)/(exp(2*summary(mod5)$solutions[3])+1) # back-transformation from z
rsup
# checking auto-correlations between samplings (less than 0.10 is good)
autocorr.diag(mod5$Sol) # for intercept
autocorr.diag(mod5$VCV) # for random effects
## creating the same model to check convergence between models
mod6=MCMCglmm(z1~1, mev=var1, random=~study+parasite_species+animal,prior=prior,pedigree=phylo3,nitt=500000,thin=250,burnin=50000, data=b)
summary(mod6)
# diagnose convergence (closer to 1 = better)
gelman.diag(mcmc.list(mod5$Sol,mod6$Sol), multivariate=F) # for the intercept
gelman.diag(mcmc.list(mod5$VCV,mod6$VCV), multivariate=F) # for the random effects
gelman.plot(mcmc.list(mod5$Sol,mod6$Sol), auto.layout = F) # to see convergence

###########################################
### expanded priors and phylogenetic matrix
prior=list(R=list(V=1,nu=0.002), G=list(G1=list(V=1,nu=1,alpha.mu=0,alpha.V =1000),G2=list(V=1,nu=1,alpha.mu=0,alpha.V =1000),G3=list(V=1,nu=1,alpha.mu=0,alpha.V =1000)))
mod7=MCMCglmm(z1~1, mev=var1, random=~study+parasite_species+animal,prior=prior,pedigree=phylo3,nitt=500000,thin=250,burnin=50000, data=b)
summary(mod7)
# back-transformation to get r (post mean and 95% CI)
rmeta=(exp(2*summary(mod7)$solutions[1])-1)/(exp(2*summary(mod7)$solutions[1])+1) # back-transformation from z
rmeta
rinf=(exp(2*summary(mod7)$solutions[2])-1)/(exp(2*summary(mod7)$solutions[2])+1) # back-transformation from z 
rinf
rsup=(exp(2*summary(mod7)$solutions[3])-1)/(exp(2*summary(mod7)$solutions[3])+1) # back-transformation from z 
rsup
# checking auto-correlations between samplings (less than 0.10 is good)
autocorr.diag(mod7$Sol) # for intercept
autocorr.diag(mod7$VCV) # for random effects
## creating the same model to check convergence between models
mod8=MCMCglmm(z1~1, mev=var1, random=~study+parasite_species+animal,prior=prior,pedigree=phylo3,nitt=500000,thin=250,burnin=50000, data=b)
summary(mod8)
# diagnose convergence (closer to 1 = better)
gelman.diag(mcmc.list(mod7$Sol,mod8$Sol), multivariate=F) # for the intercept
gelman.diag(mcmc.list(mod7$VCV,mod8$VCV), multivariate=F) # for the random effects
gelman.plot(mcmc.list(mod7$Sol,mod8$Sol), auto.layout = F) # to see convergence
# checking convergence between models with different priors (non-informative and expanded)
gelman.diag(mcmc.list(mod5$Sol,mod7$Sol), multivariate=F) # for the intercept
gelman.diag(mcmc.list(mod5$VCV,mod7$VCV), multivariate=F) # for the random effects
gelman.plot(mcmc.list(mod5$Sol,mod7$Sol), auto.layout = F) # to see convergence


### Calculation of heterogeneity (I) for phylogenetically-corrected model (non-inform priors)
# weight and mean variance
Weight<-1/var1
MV<-sum(Weight*(length(Weight)-1))/(sum(Weight)^2-sum(Weight^2))
# I at the parasite species level
I2sp<-100*(mod5$VCV[,"parasite_species"])/(mod5$VCV[,"parasite_species"]+mod5$VCV[,"study"]+mod5$VCV[,"units"]+mod5$VCV[,"animal"]+MV)
summary(I2sp)
# I at the parasite phylogeny level
I2phy<-100*(mod5$VCV[,"animal"])/(mod5$VCV[,"animal"]+mod5$VCV[,"study"]+mod5$VCV[,"units"]+mod5$VCV[,"parasite_species"]+MV)
summary(I2phy)
# I at the study level
I2st<-100*(mod5$VCV[,"study"])/(mod5$VCV[,"parasite_species"]+mod5$VCV[,"study"]+mod5$VCV[,"units"]+mod5$VCV[,"animal"]+MV)
summary(I2st)
# I at the residual level
I2res<-100*(mod5$VCV[,"units"])/(mod5$VCV[,"parasite_species"]+mod5$VCV[,"study"]+mod5$VCV[,"units"]+mod5$VCV[,"animal"]+MV)
summary(I2res)
# I total
I2t<-100*(mod5$VCV[,"units"]+mod5$VCV[,"parasite_species"]+mod5$VCV[,"study"]+mod5$VCV[,"animal"])/(mod5$VCV[,"parasite_species"]+mod5$VCV[,"study"]+mod5$VCV[,"units"]+mod5$VCV[,"animal"]+MV)
summary(I2t)


######################
# 3) META-REGRESSION #
######################
### all models with non-informative priors and with phylogenetic matrix
# global model (all fixed effects)
MCMCglmm.updateable<-updateable(MCMCglmm) # updates dredge (MuMin package) function to do models selection with bayesian models
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002),R3=list(V=1,nu=0.002),R4=list(V=1,nu=0.002),R5=list(V=1,nu=0.002),R6=list(V=1,nu=0.002)),G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod9=MCMCglmm.updateable(z1~category+n1+year+infection_type+parasite_maturity,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree=phylo3,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod9)
# heterogeneity
Weight<-1/var1
MV<-sum(Weight*(length(Weight)-1))/(sum(Weight)^2-sum(Weight^2))
# I at the parasite species level
I2sp<-100*(mod9$VCV[,"parasite_species"])/(mod9$VCV[,"parasite_species"]+mod9$VCV[,"study"]+mod9$VCV[,"units"]+mod9$VCV[,"animal"]+MV)
summary(I2sp)
# I at the parasite phylogeny level
I2phy<-100*(mod9$VCV[,"animal"])/(mod9$VCV[,"animal"]+mod9$VCV[,"study"]+mod9$VCV[,"units"]+mod9$VCV[,"parasite_species"]+MV)
summary(I2phy)
# I at the study level
I2st<-100*(mod9$VCV[,"study"])/(mod9$VCV[,"parasite_species"]+mod9$VCV[,"study"]+mod9$VCV[,"units"]+mod9$VCV[,"animal"]+MV)
summary(I2st)
# I at the residual level
I2res<-100*(mod9$VCV[,"units"])/(mod9$VCV[,"parasite_species"]+mod9$VCV[,"study"]+mod9$VCV[,"units"]+mod9$VCV[,"animal"]+MV)
summary(I2res)
# I total
I2t<-100*(mod9$VCV[,"units"]+mod9$VCV[,"parasite_species"]+mod9$VCV[,"study"]+mod9$VCV[,"animal"])/(mod9$VCV[,"parasite_species"]+mod9$VCV[,"study"]+mod9$VCV[,"units"]+mod9$VCV[,"animal"]+MV)
summary(I2t)

## models selection based on DIC
modd=dredge(mod9,rank="DIC",evaluate=T) # DIC is the bayesian equivalent for AIC 
modd # table with models ranked by values of DIC (best model=lowest DIC)

## analyzing the consensus best model
# best model's fixed effects entered according to dredge's table
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002),R3=list(V=1,nu=0.002),R4=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod10=MCMCglmm(z1~category+n1+parasite_maturity,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree=phylo3,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod10)
# heterogeneity
Weight<-1/var1
MV<-sum(Weight*(length(Weight)-1))/(sum(Weight)^2-sum(Weight^2))
# I at the parasite species level
I2sp<-100*(mod10$VCV[,"parasite_species"])/(mod10$VCV[,"parasite_species"]+mod10$VCV[,"study"]+mod10$VCV[,"units"]+mod10$VCV[,"animal"]+MV)
summary(I2sp)
# I at the parasite phylogeny level
I2phy<-100*(mod10$VCV[,"animal"])/(mod10$VCV[,"animal"]+mod10$VCV[,"study"]+mod10$VCV[,"units"]+mod10$VCV[,"parasite_species"]+MV)
summary(I2phy)
# I at the study level
I2st<-100*(mod10$VCV[,"study"])/(mod10$VCV[,"parasite_species"]+mod10$VCV[,"study"]+mod10$VCV[,"units"]+mod10$VCV[,"animal"]+MV)
summary(I2st)
# I at the residual level
I2res<-100*(mod10$VCV[,"units"])/(mod10$VCV[,"parasite_species"]+mod10$VCV[,"study"]+mod10$VCV[,"units"]+mod10$VCV[,"animal"]+MV)
summary(I2res)
# I total
I2t<-100*(mod10$VCV[,"units"]+mod10$VCV[,"parasite_species"]+mod10$VCV[,"study"]+mod10$VCV[,"animal"])/(mod10$VCV[,"parasite_species"]+mod10$VCV[,"study"]+mod10$VCV[,"units"]+mod10$VCV[,"animal"]+MV)
summary(I2t)

## significance of moderators
## publication year
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod_year=MCMCglmm(z1~year,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree=phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod_year)
# heterogeneity
Weight<-1/var1
MV<-sum(Weight*(length(Weight)-1))/(sum(Weight)^2-sum(Weight^2))
# I at the parasite species level
I2sp<-100*(mod_year$VCV[,"parasite_species"])/(mod_year$VCV[,"parasite_species"]+mod_year$VCV[,"study"]+mod_year$VCV[,"units"]+mod_year$VCV[,"animal"]+MV)
summary(I2sp)
# I at the parasite phylogeny level
I2phy<-100*(mod_year$VCV[,"animal"])/(mod_year$VCV[,"animal"]+mod_year$VCV[,"study"]+mod_year$VCV[,"units"]+mod_year$VCV[,"parasite_species"]+MV)
summary(I2phy)
# I at the study level
I2st<-100*(mod_year$VCV[,"study"])/(mod_year$VCV[,"parasite_species"]+mod_year$VCV[,"study"]+mod_year$VCV[,"units"]+mod_year$VCV[,"animal"]+MV)
summary(I2st)
# I at the residual level
I2res<-100*(mod_year$VCV[,"units"])/(mod_year$VCV[,"parasite_species"]+mod_year$VCV[,"study"]+mod_year$VCV[,"units"]+mod_year$VCV[,"animal"]+MV)
summary(I2res)
# I total
I2t<-100*(mod_year$VCV[,"units"]+mod_year$VCV[,"parasite_species"]+mod_year$VCV[,"study"]+mod_year$VCV[,"animal"])/(mod_year$VCV[,"parasite_species"]+mod_year$VCV[,"study"]+mod_year$VCV[,"units"]+mod_year$VCV[,"animal"]+MV)
summary(I2t)

## category of traits
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod_cat=MCMCglmm(z1~category,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree=phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod_cat)
# heterogeneity
Weight<-1/var1
MV<-sum(Weight*(length(Weight)-1))/(sum(Weight)^2-sum(Weight^2))
# I at the parasite species level
I2sp<-100*(mod_cat$VCV[,"parasite_species"])/(mod_cat$VCV[,"parasite_species"]+mod_cat$VCV[,"study"]+mod_cat$VCV[,"units"]+mod_cat$VCV[,"animal"]+MV)
summary(I2sp)
# I at the parasite phylogeny level
I2phy<-100*(mod_cat$VCV[,"animal"])/(mod_cat$VCV[,"animal"]+mod_cat$VCV[,"study"]+mod_cat$VCV[,"units"]+mod_cat$VCV[,"parasite_species"]+MV)
summary(I2phy)
# I at the study level
I2st<-100*(mod_cat$VCV[,"study"])/(mod_cat$VCV[,"parasite_species"]+mod_cat$VCV[,"study"]+mod_cat$VCV[,"units"]+mod_cat$VCV[,"animal"]+MV)
summary(I2st)
# I at the residual level
I2res<-100*(mod_cat$VCV[,"units"])/(mod_cat$VCV[,"parasite_species"]+mod_cat$VCV[,"study"]+mod_cat$VCV[,"units"]+mod_cat$VCV[,"animal"]+MV)
summary(I2res)
# I total
I2t<-100*(mod_cat$VCV[,"units"]+mod_cat$VCV[,"parasite_species"]+mod_cat$VCV[,"study"]+mod_cat$VCV[,"animal"])/(mod_cat$VCV[,"parasite_species"]+mod_cat$VCV[,"study"]+mod_cat$VCV[,"units"]+mod_cat$VCV[,"animal"]+MV)
summary(I2t)

## class of traits
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod_class=MCMCglmm(z1~class,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree=phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod_class)
# heterogeneity
Weight<-1/var1
MV<-sum(Weight*(length(Weight)-1))/(sum(Weight)^2-sum(Weight^2))
# I at the parasite species level
I2sp<-100*(mod_class$VCV[,"parasite_species"])/(mod_class$VCV[,"parasite_species"]+mod_class$VCV[,"study"]+mod_class$VCV[,"units"]+mod_class$VCV[,"animal"]+MV)
summary(I2sp)
# I at the parasite phylogeny level
I2phy<-100*(mod_class$VCV[,"animal"])/(mod_class$VCV[,"animal"]+mod_class$VCV[,"study"]+mod_class$VCV[,"units"]+mod_class$VCV[,"parasite_species"]+MV)
summary(I2phy)
# I at the study level
I2st<-100*(mod_class$VCV[,"study"])/(mod_class$VCV[,"parasite_species"]+mod_class$VCV[,"study"]+mod_class$VCV[,"units"]+mod_class$VCV[,"animal"]+MV)
summary(I2st)
# I at the residual level
I2res<-100*(mod_class$VCV[,"units"])/(mod_class$VCV[,"parasite_species"]+mod_class$VCV[,"study"]+mod_class$VCV[,"units"]+mod_class$VCV[,"animal"]+MV)
summary(I2res)
# I total
I2t<-100*(mod_class$VCV[,"units"]+mod_class$VCV[,"parasite_species"]+mod_class$VCV[,"study"]+mod_class$VCV[,"animal"])/(mod_class$VCV[,"parasite_species"]+mod_class$VCV[,"study"]+mod_class$VCV[,"units"]+mod_class$VCV[,"animal"]+MV)
summary(I2t)

## sampling effort
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod_samp=MCMCglmm(z1~n1,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree=phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod_samp)
# heterogeneity
Weight<-1/var1
MV<-sum(Weight*(length(Weight)-1))/(sum(Weight)^2-sum(Weight^2))
# I at the parasite species level
I2sp<-100*(mod_samp$VCV[,"parasite_species"])/(mod_samp$VCV[,"parasite_species"]+mod_samp$VCV[,"study"]+mod_samp$VCV[,"units"]+mod_samp$VCV[,"animal"]+MV)
summary(I2sp)
# I at the parasite phylogeny level
I2phy<-100*(mod_samp$VCV[,"animal"])/(mod_samp$VCV[,"animal"]+mod_samp$VCV[,"study"]+mod_samp$VCV[,"units"]+mod_samp$VCV[,"parasite_species"]+MV)
summary(I2phy)
# I at the study level
I2st<-100*(mod_samp$VCV[,"study"])/(mod_samp$VCV[,"parasite_species"]+mod_samp$VCV[,"study"]+mod_samp$VCV[,"units"]+mod_samp$VCV[,"animal"]+MV)
summary(I2st)
# I at the residual level
I2res<-100*(mod_samp$VCV[,"units"])/(mod_samp$VCV[,"parasite_species"]+mod_samp$VCV[,"study"]+mod_samp$VCV[,"units"]+mod_samp$VCV[,"animal"]+MV)
summary(I2res)
# I total
I2t<-100*(mod_samp$VCV[,"units"]+mod_samp$VCV[,"parasite_species"]+mod_samp$VCV[,"study"]+mod_samp$VCV[,"animal"])/(mod_samp$VCV[,"parasite_species"]+mod_samp$VCV[,"study"]+mod_samp$VCV[,"units"]+mod_samp$VCV[,"animal"]+MV)
summary(I2t)

## infection type
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod_inf=MCMCglmm(z1~infection_type,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree=phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod_inf)
# heterogeneity
Weight<-1/var1
MV<-sum(Weight*(length(Weight)-1))/(sum(Weight)^2-sum(Weight^2))
# I at the parasite species level
I2sp<-100*(mod_inf$VCV[,"parasite_species"])/(mod_inf$VCV[,"parasite_species"]+mod_inf$VCV[,"study"]+mod_inf$VCV[,"units"]+mod_inf$VCV[,"animal"]+MV)
summary(I2sp)
# I at the parasite phylogeny level
I2phy<-100*(mod_inf$VCV[,"animal"])/(mod_inf$VCV[,"animal"]+mod_inf$VCV[,"study"]+mod_inf$VCV[,"units"]+mod_inf$VCV[,"parasite_species"]+MV)
summary(I2phy)
# I at the study level
I2st<-100*(mod_inf$VCV[,"study"])/(mod_inf$VCV[,"parasite_species"]+mod_inf$VCV[,"study"]+mod_inf$VCV[,"units"]+mod_inf$VCV[,"animal"]+MV)
summary(I2st)
# I at the residual level
I2res<-100*(mod_inf$VCV[,"units"])/(mod_inf$VCV[,"parasite_species"]+mod_inf$VCV[,"study"]+mod_inf$VCV[,"units"]+mod_inf$VCV[,"animal"]+MV)
summary(I2res)
# I total
I2t<-100*(mod_inf$VCV[,"units"]+mod_inf$VCV[,"parasite_species"]+mod_inf$VCV[,"study"]+mod_inf$VCV[,"animal"])/(mod_inf$VCV[,"parasite_species"]+mod_inf$VCV[,"study"]+mod_inf$VCV[,"units"]+mod_inf$VCV[,"animal"]+MV)
summary(I2t)

## parasite maturity
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod_mat=MCMCglmm(z1~parasite_maturity,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree=phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod_mat)
# heterogeneity
Weight<-1/var1
MV<-sum(Weight*(length(Weight)-1))/(sum(Weight)^2-sum(Weight^2))
# I at the parasite species level
I2sp<-100*(mod_mat$VCV[,"parasite_species"])/(mod_mat$VCV[,"parasite_species"]+mod_mat$VCV[,"study"]+mod_mat$VCV[,"units"]+mod_mat$VCV[,"animal"]+MV)
summary(I2sp)
# I at the parasite phylogeny level
I2phy<-100*(mod_mat$VCV[,"animal"])/(mod_mat$VCV[,"animal"]+mod_mat$VCV[,"study"]+mod_mat$VCV[,"units"]+mod_mat$VCV[,"parasite_species"]+MV)
summary(I2phy)
# I at the study level
I2st<-100*(mod_mat$VCV[,"study"])/(mod_mat$VCV[,"parasite_species"]+mod_mat$VCV[,"study"]+mod_mat$VCV[,"units"]+mod_mat$VCV[,"animal"]+MV)
summary(I2st)
# I at the residual level
I2res<-100*(mod_mat$VCV[,"units"])/(mod_mat$VCV[,"parasite_species"]+mod_mat$VCV[,"study"]+mod_mat$VCV[,"units"]+mod_mat$VCV[,"animal"]+MV)
summary(I2res)
# I total
I2t<-100*(mod_mat$VCV[,"units"]+mod_mat$VCV[,"parasite_species"]+mod_mat$VCV[,"study"]+mod_mat$VCV[,"animal"])/(mod_mat$VCV[,"parasite_species"]+mod_mat$VCV[,"study"]+mod_mat$VCV[,"units"]+mod_mat$VCV[,"animal"]+MV)
summary(I2t)

## mean estimate for each factor level
## category of traits
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod_cat2=MCMCglmm(z1~factor(category)-1,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree = phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod_cat2)
rmeta_b=(exp(2*summary(mod_cat2)$solutions[1])-1)/(exp(2*summary(mod_cat2)$solutions[1])+1) # back-transformation from z 
rmeta_b
rinf_b=(exp(2*summary(mod_cat2)$solutions[6])-1)/(exp(2*summary(mod_cat2)$solutions[6])+1) # back-transformation from z 
rinf_b
rsup_b=(exp(2*summary(mod_cat2)$solutions[11])-1)/(exp(2*summary(mod_cat2)$solutions[11])+1) # back-transformation from z
rsup_b
rmeta_l=(exp(2*summary(mod_cat2)$solutions[2])-1)/(exp(2*summary(mod_cat2)$solutions[2])+1) # back-transformation from z 
rmeta_l
rinf_l=(exp(2*summary(mod_cat2)$solutions[7])-1)/(exp(2*summary(mod_cat2)$solutions[7])+1) # back-transformation from z 
rinf_l
rsup_l=(exp(2*summary(mod_cat2)$solutions[12])-1)/(exp(2*summary(mod_cat2)$solutions[12])+1) # back-transformation from z 
rsup_l
rmeta_m=(exp(2*summary(mod_cat2)$solutions[3])-1)/(exp(2*summary(mod_cat2)$solutions[3])+1) # back-transformation from z 
rmeta_m
rinf_m=(exp(2*summary(mod_cat2)$solutions[8])-1)/(exp(2*summary(mod_cat2)$solutions[8])+1) # back-transformation from z 
rinf_m
rsup_m=(exp(2*summary(mod_cat2)$solutions[13])-1)/(exp(2*summary(mod_cat2)$solutions[13])+1) # back-transformation from z 
rsup_m
rmeta_ph=(exp(2*summary(mod_cat2)$solutions[4])-1)/(exp(2*summary(mod_cat2)$solutions[4])+1) # back-transformation from z 
rmeta_ph
rinf_ph=(exp(2*summary(mod_cat2)$solutions[9])-1)/(exp(2*summary(mod_cat2)$solutions[9])+1) # back-transformation from z 
rinf_ph
rsup_ph=(exp(2*summary(mod_cat2)$solutions[14])-1)/(exp(2*summary(mod_cat2)$solutions[14])+1) # back-transformation from z 
rsup_ph
rmeta_pr=(exp(2*summary(mod_cat2)$solutions[5])-1)/(exp(2*summary(mod_cat2)$solutions[5])+1) # back-transformation from z 
rmeta_pr
rinf_pr=(exp(2*summary(mod_cat2)$solutions[10])-1)/(exp(2*summary(mod_cat2)$solutions[10])+1) # back-transformation from z 
rinf_pr
rsup_pr=(exp(2*summary(mod_cat2)$solutions[15])-1)/(exp(2*summary(mod_cat2)$solutions[15])+1) # back-transformation from z 
rsup_pr

## parasite maturity
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod_mat2=MCMCglmm(z1~factor(parasite_maturity)-1,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree = phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod_mat2)
rmeta_ac=(exp(2*summary(mod_mat2)$solutions[1])-1)/(exp(2*summary(mod_mat2)$solutions[1])+1) # back-transformation from z 
rmeta_ac
rinf_ac=(exp(2*summary(mod_mat2)$solutions[3])-1)/(exp(2*summary(mod_mat2)$solutions[3])+1) # back-transformation from z 
rinf_ac
rsup_ac=(exp(2*summary(mod_mat2)$solutions[5])-1)/(exp(2*summary(mod_mat2)$solutions[5])+1) # back-transformation from z 
rsup_ac
rmeta_cy=(exp(2*summary(mod_mat2)$solutions[2])-1)/(exp(2*summary(mod_mat2)$solutions[2])+1) # back-transformation from z 
rmeta_cy
rinf_cy=(exp(2*summary(mod_mat2)$solutions[4])-1)/(exp(2*summary(mod_mat2)$solutions[4])+1) # back-transformation from z
rinf_cy
rsup_cy=(exp(2*summary(mod_mat2)$solutions[6])-1)/(exp(2*summary(mod_mat2)$solutions[6])+1) # back-transformation from z 
rsup_cy

## infection type
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod_inf2=MCMCglmm(z1~factor(infection_type)-1,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree = phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod_inf2)
rmeta_ex=(exp(2*summary(mod_inf2)$solutions[1])-1)/(exp(2*summary(mod_inf2)$solutions[1])+1) # back-transformation from z 
rmeta_ex
rinf_ex=(exp(2*summary(mod_inf2)$solutions[3])-1)/(exp(2*summary(mod_inf2)$solutions[3])+1) # back-transformation from z 
rinf_ex
rsup_ex=(exp(2*summary(mod_inf2)$solutions[5])-1)/(exp(2*summary(mod_inf2)$solutions[5])+1) # back-transformation from z 
rsup_ex
rmeta_na=(exp(2*summary(mod_inf2)$solutions[2])-1)/(exp(2*summary(mod_inf2)$solutions[2])+1) # back-transformation from z 
rmeta_na
rinf_na=(exp(2*summary(mod_inf2)$solutions[4])-1)/(exp(2*summary(mod_inf2)$solutions[4])+1) # back-transformation from z 
rinf_na
rsup_na=(exp(2*summary(mod_inf2)$solutions[6])-1)/(exp(2*summary(mod_inf2)$solutions[6])+1) # back-transformation from z 
rsup_na

## class of traits
prior=list(R=list(R1=list(V=1,nu=0.002),R2=list(V=1,nu=0.002)), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod_class2=MCMCglmm(z1~factor(class)-1,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree = phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod_class2)
rmeta_act=(exp(2*summary(mod_class2)$solutions[1])-1)/(exp(2*summary(mod_class2)$solutions[1])+1) # back-transformation from z 
rmeta_act
rinf_act=(exp(2*summary(mod_class2)$solutions[17])-1)/(exp(2*summary(mod_class2)$solutions[17])+1) # back-transformation from z 
rinf_act
rsup_act=(exp(2*summary(mod_class2)$solutions[33])-1)/(exp(2*summary(mod_class2)$solutions[33])+1) # back-transformation from z 
rsup_act
rmeta_bo=(exp(2*summary(mod_class2)$solutions[2])-1)/(exp(2*summary(mod_class2)$solutions[2])+1) # back-transformation from z
rmeta_bo
rinf_bo=(exp(2*summary(mod_class2)$solutions[18])-1)/(exp(2*summary(mod_class2)$solutions[18])+1) # back-transformation from z 
rinf_bo
rsup_bo=(exp(2*summary(mod_class2)$solutions[34])-1)/(exp(2*summary(mod_class2)$solutions[34])+1) # back-transformation from z 
rsup_bo
rmeta_co=(exp(2*summary(mod_class2)$solutions[3])-1)/(exp(2*summary(mod_class2)$solutions[3])+1) # back-transformation from z 
rmeta_co
rinf_co=(exp(2*summary(mod_class2)$solutions[19])-1)/(exp(2*summary(mod_class2)$solutions[19])+1) # back-transformation from z 
rinf_co
rsup_co=(exp(2*summary(mod_class2)$solutions[35])-1)/(exp(2*summary(mod_class2)$solutions[35])+1) # back-transformation from z 
rsup_co
rmeta_fo=(exp(2*summary(mod_class2)$solutions[4])-1)/(exp(2*summary(mod_class2)$solutions[4])+1) # back-transformation from z 
rmeta_fo
rinf_fo=(exp(2*summary(mod_class2)$solutions[20])-1)/(exp(2*summary(mod_class2)$solutions[20])+1) # back-transformation from z 
rinf_fo
rsup_fo=(exp(2*summary(mod_class2)$solutions[36])-1)/(exp(2*summary(mod_class2)$solutions[36])+1) # back-transformation from z 
rsup_fo
rmeta_gr=(exp(2*summary(mod_class2)$solutions[5])-1)/(exp(2*summary(mod_class2)$solutions[5])+1) # back-transformation from z 
rmeta_gr
rinf_gr=(exp(2*summary(mod_class2)$solutions[21])-1)/(exp(2*summary(mod_class2)$solutions[21])+1) # back-transformation from z 
rinf_gr
rsup_gr=(exp(2*summary(mod_class2)$solutions[37])-1)/(exp(2*summary(mod_class2)$solutions[37])+1) # back-transformation from z 
rsup_gr
rmeta_im=(exp(2*summary(mod_class2)$solutions[6])-1)/(exp(2*summary(mod_class2)$solutions[6])+1) # back-transformation from z 
rmeta_im
rinf_im=(exp(2*summary(mod_class2)$solutions[22])-1)/(exp(2*summary(mod_class2)$solutions[22])+1) # back-transformation from z
rinf_im
rsup_im=(exp(2*summary(mod_class2)$solutions[38])-1)/(exp(2*summary(mod_class2)$solutions[38])+1) # back-transformation from z 
rsup_im
rmeta_me=(exp(2*summary(mod_class2)$solutions[7])-1)/(exp(2*summary(mod_class2)$solutions[7])+1) # back-transformation from z 
rmeta_me
rinf_me=(exp(2*summary(mod_class2)$solutions[23])-1)/(exp(2*summary(mod_class2)$solutions[23])+1) # back-transformation from z 
rinf_me
rsup_me=(exp(2*summary(mod_class2)$solutions[39])-1)/(exp(2*summary(mod_class2)$solutions[39])+1) # back-transformation from z 
rsup_me
rmeta_ne=(exp(2*summary(mod_class2)$solutions[8])-1)/(exp(2*summary(mod_class2)$solutions[8])+1) # back-transformation from z 
rmeta_ne
rinf_ne=(exp(2*summary(mod_class2)$solutions[24])-1)/(exp(2*summary(mod_class2)$solutions[24])+1) # back-transformation from z 
rinf_ne
rsup_ne=(exp(2*summary(mod_class2)$solutions[40])-1)/(exp(2*summary(mod_class2)$solutions[40])+1) # back-transformation from z 
rsup_ne
rmeta_nh=(exp(2*summary(mod_class2)$solutions[9])-1)/(exp(2*summary(mod_class2)$solutions[9])+1) # back-transformation from z
rmeta_nh
rinf_nh=(exp(2*summary(mod_class2)$solutions[25])-1)/(exp(2*summary(mod_class2)$solutions[25])+1) # back-transformation from z 
rinf_nh
rsup_nh=(exp(2*summary(mod_class2)$solutions[41])-1)/(exp(2*summary(mod_class2)$solutions[41])+1) # back-transformation from z 
rsup_nh
rmeta_pro=(exp(2*summary(mod_class2)$solutions[10])-1)/(exp(2*summary(mod_class2)$solutions[10])+1) # back-transformation from z 
rmeta_pro
rinf_pro=(exp(2*summary(mod_class2)$solutions[26])-1)/(exp(2*summary(mod_class2)$solutions[26])+1) # back-transformation from z 
rinf_pro
rsup_pro=(exp(2*summary(mod_class2)$solutions[42])-1)/(exp(2*summary(mod_class2)$solutions[42])+1) # back-transformation from z 
rsup_pro
rmeta_re=(exp(2*summary(mod_class2)$solutions[11])-1)/(exp(2*summary(mod_class2)$solutions[11])+1) # back-transformation from z 
rmeta_re
rinf_re=(exp(2*summary(mod_class2)$solutions[27])-1)/(exp(2*summary(mod_class2)$solutions[27])+1) # back-transformation from z
rinf_re
rsup_re=(exp(2*summary(mod_class2)$solutions[43])-1)/(exp(2*summary(mod_class2)$solutions[43])+1) # back-transformation from z 
rsup_re
rmeta_so=(exp(2*summary(mod_class2)$solutions[12])-1)/(exp(2*summary(mod_class2)$solutions[12])+1) # back-transformation from z 
rmeta_so
rinf_so=(exp(2*summary(mod_class2)$solutions[28])-1)/(exp(2*summary(mod_class2)$solutions[28])+1) # back-transformation from z 
rinf_so
rsup_so=(exp(2*summary(mod_class2)$solutions[44])-1)/(exp(2*summary(mod_class2)$solutions[44])+1) # back-transformation from z
rsup_so
rmeta_st=(exp(2*summary(mod_class2)$solutions[13])-1)/(exp(2*summary(mod_class2)$solutions[13])+1) # back-transformation from z 
rmeta_st
rinf_st=(exp(2*summary(mod_class2)$solutions[19])-1)/(exp(2*summary(mod_class2)$solutions[29])+1) # back-transformation from z 
rinf_st
rsup_st=(exp(2*summary(mod_class2)$solutions[45])-1)/(exp(2*summary(mod_class2)$solutions[45])+1) # back-transformation from z 
rsup_st
rmeta_sh=(exp(2*summary(mod_class2)$solutions[14])-1)/(exp(2*summary(mod_class2)$solutions[14])+1) # back-transformation from z 
rmeta_sh
rinf_sh=(exp(2*summary(mod_class2)$solutions[30])-1)/(exp(2*summary(mod_class2)$solutions[30])+1) # back-transformation from z 
rinf_sh
rsup_sh=(exp(2*summary(mod_class2)$solutions[46])-1)/(exp(2*summary(mod_class2)$solutions[46])+1) # back-transformation from z 
rsup_sh
rmeta_su=(exp(2*summary(mod_class2)$solutions[15])-1)/(exp(2*summary(mod_class2)$solutions[15])+1) # back-transformation from z 
rmeta_su
rinf_su=(exp(2*summary(mod_class2)$solutions[31])-1)/(exp(2*summary(mod_class2)$solutions[31])+1) # back-transformation from z 
rinf_su
rsup_su=(exp(2*summary(mod_class2)$solutions[47])-1)/(exp(2*summary(mod_class2)$solutions[47])+1) # back-transformation from z 
rsup_su
rmeta_ta=(exp(2*summary(mod_class2)$solutions[16])-1)/(exp(2*summary(mod_class2)$solutions[16])+1) # back-transformation from z 
rmeta_ta
rinf_ta=(exp(2*summary(mod_class2)$solutions[32])-1)/(exp(2*summary(mod_class2)$solutions[32])+1) # back-transformation from z 
rinf_ta
rsup_ta=(exp(2*summary(mod_class2)$solutions[48])-1)/(exp(2*summary(mod_class2)$solutions[48])+1) # back-transformation from z 
rsup_ta


#######################
# 4) PUBLICATION BIAS #
#######################
# meta-analysis model without fixed effects (non-inform. priors and phylo matrix)
prior=list(R=list(V=1,nu=0.002), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod5=MCMCglmm(z1~1,mev=var1,random=~study+parasite_species+animal,prior=prior,pedigree=phylo3,pr=T,nitt=500000,thin=250,burnin=50000, data=b)
summary(mod5)
sink("publication bias1.txt")
print(summary(mod5))
sink()
# reorganizing the syntax
mod5$Random$formula <- update(mod5$Random$formula, ~.+leg(mev, -1, FALSE):units)
# getting predictions (raw data - predictions = meta-analytic residuals)
# function from Jarod Hadfield to get predictions
split.direct.sum<-function(x){  
  
  if(is.na(x)){
    return(NULL)
  }else{
    x <- gsub("\n", "", x, fixed=TRUE)  # required if the random formula is very long and broken over lines
    openB<-gregexpr("\\(", x)[[1]]
    closeB<-gregexpr("\\)", x)[[1]]
    true_openB<-openB
    true_closeB<-closeB
    
    for(i in 1:length(openB)){
      dist<-outer(openB, closeB, function(x,y){y-x})
      dist[which(dist<0)]<-Inf
      dist<-which(dist==min(dist), arr.ind=T)[1,] 
      true_openB[i]<-openB[dist[1]]
      true_closeB[i]<-closeB[dist[2]]
      openB<-openB[-dist[1]]
      closeB<-closeB[-dist[2]]
    }
    
    plus<-gregexpr("\\+", x)[[1]]
    internals<-matrix(mapply(function(x,y){(plus>x & plus<y)}, x=true_openB, y=true_closeB), length(plus), length(true_openB))
    rterms<-strsplit(x, "")[[1]]
    rterms[plus[which(rowSums(internals)!=0)]]<-"leaveMCMCleave"
    rterms<-paste(rterms, collapse="")
    rterms<-strsplit(rterms, " *\\+ *")[[1]]
    rterms<-gsub("leaveMCMCleave", "+", rterms)
    return(rterms)
  }
}
predict.MCMCglmm<-function(object, newdata=NULL, marginal=object$Random$formula, type="response", interval="none", level=0.95, it=NULL, posterior="all", verbose=FALSE, approx="numerical", ...){
  
  rm.obs<-c()
  
  if(interval=="prediction"){
    post.pred<-t(simulate(object=object, nsim=nrow(object$Sol), newdata=newdata, marginal=marginal, type=type, it=it, posterior=posterior, verbose=verbose))
    
  }else{
    
    if(type%in%c("response", "terms")==FALSE){stop("type must be response or terms")}
    if(interval%in%c("none", "confidence", "prediction")==FALSE){stop("interval must be none, confidence or prediction")}
    
    if(!is.null(posterior)){
      if(!posterior%in%c("distribution", "mean", "mode", "all")){
        stop("posterior argument must be either distribution, mean, mode or all")
      }
    }
    
    if(!is.null(marginal)){
      if(class(marginal)!="formula"){stop("marginal should be NULL or a formula")}
    }
    
    if(!is.null(it)){
      if(length(it)>1){stop("it should be an integer")}
      if(it>nrow(object$Sol) | it<1){stop("it should be less than or equal to the number of iterations")}
    }
    
    rcomponents<-split.direct.sum(as.character(object$Random$formula)[2])
    mcomponents<-split.direct.sum(as.character(marginal)[2])
    
    if(is.null(object$meta)){object$meta<-FALSE}
    
    if(any(mcomponents%in%rcomponents==FALSE)){stop("marginal formula does not correspond to model formula")}
    
    marginalise<-rep(as.numeric(rcomponents%in%mcomponents), object$Random$nrt)
    
    if(!is.null(newdata)){
      suppressWarnings(object2<-MCMCglmm(fixed=object$Fixed$formula, random=object$Random$formula, rcov=object$Residual$formula, family=object$Residual$original.family, data=newdata, nitt=1, thin=1, burnin=0, ginverse=object$ginverse, verbose=FALSE, pr=any(marginalise==0)))
      find.fixed<-match(colnames(object2$Sol)[1:object2$Fixed$nfl], colnames(object$Sol))
      find.random<-match(colnames(object2$Sol)[-c(1:object2$Fixed$nfl)], colnames(object$Sol))
      if(any(is.na(find.fixed))){stop("model for newdata has fixed effects not present in original model")}
      if(verbose){
        if(any(is.na(find.random))){
          missing.random<-colnames(object2$Sol)[which(is.na(find.random))+object2$Fixed$nfl]
          warning(paste("model for newdata has random effects not present in original model:", paste(missing.random, collapse=", ")))
        }
        missing.fixed<-which(!colnames(object$Sol)[1:object$Fixed$nfl]%in%colnames(object2$Sol)[1:object2$Fixed$nfl])
        if(length(missing.fixed)>0){
          missing.fixed<-colnames(object$Sol)[1:object$Fixed$nfl][missing.fixed]
          warning(paste("original model has fixed effects not present in newdata:", paste(missing.fixed, collapse=", ")))
        }
      }
      object2$Sol<-object$Sol[,c(find.fixed, find.random),drop=FALSE]
      find.vcv<-match(colnames(object2$VCV), colnames(object$VCV))
      if(any(is.na(find.vcv))){stop("model for newdata has (co)variance terms not in original model")}
      object2$VCV<-object$VCV[,find.vcv,drop=FALSE]
      if(!is.null(object2$CP)){
        find.cp<-match(colnames(object2$CP), colnames(object$CP))
        if(any(is.na(find.cp))){stop("model for newdata has cutpoints not in original model")}
        object2$CP<-object$CP[,find.cp,drop=FALSE]
      }
      object<-object2
      rm(object2)
    }
    
    if(posterior=="mean"){
      object$VCV<-matrix(colMeans(object$VCV), 1, ncol(object$VCV))
      object$Sol<-matrix(colMeans(object$Sol), 1, ncol(object$Sol))
      it<-1
    }
    if(posterior=="mode"){
      object$VCV<-matrix(posterior.mode(object$VCV, ...), 1, ncol(object$VCV))
      object$Sol<-matrix(posterior.mode(object$Sol, ...), 1, ncol(object$Sol))
      it<-1
    }
    if(is.null(it)){
      if(posterior=="distribution"){
        it<-sample(nrow(object$Sol), 1)
      }
      if(posterior=="all"){
        it<-1:nrow(object$Sol)
      }
    }
    object$Sol<-object$Sol[it,,drop=FALSE]
    object$VCV<-object$VCV[it,,drop=FALSE]
    if(!is.null(object$Lambda)){
      object$Lambda<-object$Lambda[it,,drop=FALSE]
    }
    if(!is.null(object$CP)){
      object$CP<-object$CP[it,,drop=FALSE]
    }
    
    if(is.null(object$X)){
      stop("fixed effect design matrix not saved: pass saveX=TRUE to MCMCglmm")
    }
    
    if(any(marginalise==0) & dim(object$Sol)[2]==dim(object$X)[2]){
      stop("posterior distribution of random effects not saved: pass pr=TRUE to MCMCglmm")
    }
    if(any(marginalise==0) & is.null(object$Z)){
      stop("random effect design matrix not saved: pass saveZ=TRUE to MCMCglmm")
    }
    
    
    if(is.null(object$Random$nfl)==FALSE){  # there are random effects
      st<-c(1,cumsum(object$Random$nrl*object$Random$nfl)+1)  # starting column for random effects of each component 
      st<-st[-length(st)]
      end<-cumsum(object$Random$nrl*object$Random$nfl)        # ennding column for random effects of each component 
      keep<-unlist(mapply(st[which(marginalise==0)], end[which(marginalise==0)], FUN=":"))    # random effects to be kept
    }else{
      keep<-NULL
    }
    
    if(!is.null(newdata) & !is.null(object$Random$nfl)){
      missing.random<-which(is.na(object$Sol[1,]))
      if(posterior=="mean" | posterior=="mode"){
        if(any(is.na(object$Sol))){
          object$Sol[missing.random]<-0
        }
      }else{
        dv<-1:ncol(object$Sol)
        cnt<-0
        for(j in 1:length(object$Random$nfl)){
          nfl<-object$Random$nfl[j]
          nrl<-object$Random$nrl[j]
          dv[object$Fixed$nfl+cnt+1:(nfl*nrl)]<-rep(diag(matrix(1:(nfl^2),nfl,nfl)), each=nrl)
          cnt<-cnt+(nfl*nrl)
        }
        if(any(is.na(object$Sol))){
          object$Sol[,missing.random,drop=FALSE]<-rnorm(nrow(object$Sol)*length(missing.random), 0, sqrt(object$VCV[,dv[missing.random]]))
        }
      }
    }
    
    object$Sol<-object$Sol[,c(1:object$Fixed$nfl, object$Fixed$nfl+keep),drop=FALSE]
    
    W<-cBind(object$X, object$Z)
    W<-W[,c(1:object$Fixed$nfl, object$Fixed$nfl+keep), drop=FALSE]
    
    post.pred<-t(apply(object$Sol,1, function(x){(W%*%x)@x}))
    
    if(type=="response"){
      
      if(any(object$family!="gaussian" & object$family!="cengaussian")){
        post.var<-buildV(object, marginal=marginal, diag=TRUE, it=NULL, posterior="all", ...)
      }
      
      super.trait<-1:length(object$Residual$family)
      cnt<-1
      i<-1
      while(i<=length(super.trait)){
        if(!grepl("hu|zi|za|multinomial", object$Residual$family[i])){
          super.trait[i]<-cnt
          i<-i+1
          cnt<-cnt+1
        }else{
          if(grepl("multinomial", object$Residual$family[i])){
            nm<-as.numeric(substr(object$Residual$family[i], 12,nchar(object$Residual$family[i])))-1
            super.trait[i+1:nm-1]<-cnt
            i<-i+nm
            cnt<-cnt+1
          }else{
            super.trait[i]<-cnt
            super.trait[i+1]<-cnt
            i<-i+2
            cnt<-cnt+1
          }
        }
      }
      
      normal.evd<-function(mu, v, approx){
        if(approx=="numerical" || approx=="taylor2"){
          if(approx=="numerical"){ 
            int.foo<-function(x, mu, v){exp(-exp(x))*dnorm(x, mu, sqrt(v))}
            return(integrate(int.foo, qnorm(1e-6, mu,sqrt(v)), qnorm(1-1e-6, mu,sqrt(v)), mu,v)[[1]])
          }else{
            return(exp(-exp(mu))+0.5*v*exp(-exp(mu)+mu)*(exp(mu)-1))
          }
        }else{  
          stop(paste(approx, "approximation not implemented for this response"))
        }  
      }
      normal.zt<-function(mu, v, approx){
        if(approx=="numerical" || approx=="taylor2"){
          if(approx=="numerical"){
            int.foo<-function(x, mu, v){exp(x)/(1-exp(-exp(x)))*dnorm(x, mu, sqrt(v))}
            return(integrate(int.foo, qnorm(0.0001, mu,sqrt(v)), qnorm(0.9999, mu,sqrt(v)), mu,v)[[1]])
          }else{
            return(exp(mu)/(1-exp(-exp(mu)))+0.5*v*exp(exp(mu)+mu)*(1-2*exp(exp(mu))+exp(2*exp(mu))+3*exp(mu)+2*exp(2*mu)-3*exp(exp(mu)+mu)+exp(exp(mu)+2*mu))/(exp(exp(mu))-1)^3)
          } 
        }else{  
          stop(paste(approx, "approximation not implemented for this response"))
        }  
      }  
      
      normal.logistic<-function(mu, v, approx){
        if(approx=="numerical"){
          int.foo<-function(x, mu, v){plogis(x)*dnorm(x, mu, sqrt(v))}
          return(integrate(int.foo, qnorm(0.0001, mu,sqrt(v)), qnorm(0.9999, mu,sqrt(v)), mu,v)[[1]])
        }
        if(approx=="diggle"){
          return(plogis(mu/sqrt(1+v*(16*sqrt(3)/(15*pi))^2)))
        }
        if(approx=="mcculloch"){
          return(plogis(mu-0.5*v*tanh(mu*(1+2*exp(-0.5*v))/6)))
        }
        if(approx=="taylor2"){
          return(plogis(mu)-0.5*v*exp(mu)*(exp(mu)-1)/(1+exp(mu))^3)
        }
      }
      
      normal.clogistic<-function(mu, v, size, approx){
        if(approx=="numerical" || approx=="taylor2"){
          if(approx=="numerical"){
            int.foo<-function(x, mu, v, size){(1-((1-plogis(mu))^size))*dnorm(x, mu, sqrt(v))}
            return(integrate(int.foo, qnorm(0.0001, mu,sqrt(v)), qnorm(0.9999, mu,sqrt(v)), mu,v, size)[[1]])
          }else{
            return((1-((1-plogis(mu))^size))-0.5*v*exp(mu)*size*(size*exp(mu)-1)/(1+exp(mu))^(2+size))
          }
        }else{
          stop(paste(approx, "approximation not implemented for this response"))
        }  
      }
      
      normal.multilogistic<-function(mu, v, approx){
        if(approx=="numerical"){
          int.foo<-function(x, mu, v, i){(exp(x[i])/(1+sum(exp(x))))*prod(dnorm(x, mu, sqrt(v)))}
          res<-1:length(mu)
          for(i in 1:length(mu)){
            res[i]<-cubature::adaptIntegrate(int.foo, qnorm(0.0001, mu,sqrt(v)), qnorm(0.9999, mu,sqrt(v)), mu=mu, v=v, i=i)[[1]]
          }
          return(res)
        }else{
          stop(paste(approx, "approximation not implemented for this response"))
        }
      } 
      
      if(!is.null(object$Lambda)){
        if(any(object$family!="gaussian")){stop("sorry - prediction not yet available for sir models applied to non-Gaussian data")}
        # bear in mind that if this is implemented the post.var is quadratic in the structural parameters
        I<-Diagonal(nrow(object$XL))
        post.pred<-t(sapply(1:nrow(object$Sol), function(i){as.vector(solve(I-object$XL%*%kronecker(object$Lambda[i,], I), post.pred[i,]))}))
      }
      
      if(any(object$family%in%c("poisson","cenpoisson"))){
        keep<-which(object$family%in%c("poisson","cenpoisson"))
        post.pred[,keep]<-exp(post.pred[,keep]+0.5*post.var[,keep])
      }
      
      if(any(object$family%in%c("ztpoisson"))){
        keep<-which(object$family%in%c("ztpoisson"))
        post.pred[,keep]<-mapply(post.pred[,keep], post.var[,keep], FUN=function(mu,v){normal.zt(mu,v, approx)})
      }
      
      if(any(object$family%in%c("exponential","cenexponential","geometric","cengeometric"))){
        keep<-which(object$family%in%c("exponential","cenexponential","geometric","cengeometric"))
        post.pred[,keep]<-exp(-post.pred[,keep]+0.5*post.var[,keep])
      }
      
      if(any(object$family%in%c("ordinal"))){      
        
        nord<-unique(object$error.term[which(object$family=="ordinal")])
        cp.names<-substr(colnames(object$CP), 10, regexpr("\\.[1-9]$|\\.[1-9][0-9]$", colnames(object$CP))-1)
        cp.names<-match(cp.names,unique(cp.names))
        
        for(k in 1:length(nord)){
          
          keep<-which(object$family%in%c("ordinal") & object$error.term==nord[k])
          CP<-cbind(-Inf, 0, object$CP[,which(cp.names==k), drop=FALSE], Inf)
          q<-matrix(0,dim(post.pred)[1], length(keep))
          
          for(i in 2:(dim(CP)[2]-1)){
            q<-q+(pnorm(CP[,i+1]-post.pred[,keep],0,sqrt(post.var[,keep]+1))-pnorm(CP[,i]-post.pred[,keep],0,sqrt(post.var[,keep]+1)))*(i-1)
          }
          
          post.pred[,keep]<-q
          rm(q)
        } 
      }
      
      if(any(object$family%in%c("threshold"))){      
        
        nord<-unique(object$error.term[which(object$family=="threshold")])
        cp.names<-substr(colnames(object$CP), 10, regexpr("\\.[1-9]$|\\.[1-9][0-9]$", colnames(object$CP))-1)
        cp.names<-match(cp.names,unique(cp.names))
        
        for(k in 1:length(nord)){
          keep<-which(object$family%in%c("threshold") & object$error.term==nord[k])
          CP<-cbind(-Inf, 0, object$CP[,which(cp.names==k), drop=FALSE], Inf)
          q<-matrix(0,dim(post.pred)[1], length(keep))
          
          for(i in 2:(dim(CP)[2]-1)){
            q<-q+(pnorm(CP[,i+1]-post.pred[,keep],0,sqrt(post.var[,keep]))-pnorm(CP[,i]-post.pred[,keep],0,sqrt(post.var[,keep])))*(i-1)
          }
          
          post.pred[,keep]<-q
          rm(q)
        } 
      }
      
      if(any(grepl("nzbinom", object$family))){
        keep<-grep("nzbinom", object$family)
        size<-t(matrix(as.numeric(substr(object$family[keep], 8, nchar(object$family[keep]))), length(keep),nrow(post.pred)))
        post.pred[,keep]<-mapply(post.pred[,keep], post.var[,keep], size, FUN=function(mu,v,size){normal.clogistic(mu,v,size, approx)})
      }
      
      for(k in unique(super.trait)){
        if(any(grepl("multinomial", object$Residual$family))){
          keep<-which(object$error.term%in%which(super.trait==k))
          size<-as.numeric(substr(object$family[keep], 12, nchar(object$family[keep])))
          for(j in 1:nrow(post.pred)){
            prob<-matrix(post.pred[j,keep], length(keep)/sum(super.trait==k), sum(super.trait==k))
            pvar<-matrix(post.var[j,keep], length(keep)/sum(super.trait==k), sum(super.trait==k))
            post.pred[j,keep]<-t(sapply(1:nrow(prob), function(x){normal.multilogistic(prob[x,], pvar[x,], approx)}))*size
          }
        }
        
        if(any(grepl("hupoisson", object$Residual$family))){
          keep<-which(object$error.term%in%which(super.trait==k))
          keep<-keep[-c(1:(length(keep)/2))]
          rm.obs<-c(rm.obs, keep)
          post.pred[,keep]<-mapply(post.pred[,keep], post.var[,keep], FUN=function(mu,v){1-normal.logistic(mu,v, approx)})
          post.pred[,keep-length(keep)]<-post.pred[,keep]*mapply(post.pred[,keep-length(keep)], post.var[,keep-length(keep)], FUN=function(mu,v){normal.zt(mu,v, approx)})
        }
        if(any(grepl("zapoisson", object$Residual$family))){
          keep<-which(object$error.term%in%which(super.trait==k))
          keep<-keep[-c(1:(length(keep)/2))]
          rm.obs<-c(rm.obs, keep)
          post.pred[,keep]<-1-mapply(post.pred[,keep], post.var[,keep], FUN=function(mu,v){normal.evd(mu,v, approx)})
          post.pred[,keep-length(keep)]<-post.pred[,keep]*mapply(post.pred[,keep-length(keep)], post.var[,keep-length(keep)], FUN=function(mu,v){normal.zt(mu,v, approx)})
        }
        
        if(any(grepl("zipoisson", object$Residual$family[which(super.trait==k)]))){
          keep<-which(object$error.term%in%which(super.trait==k))
          keep<-keep[-c(1:(length(keep)/2))]
          rm.obs<-c(rm.obs, keep)
          post.pred[,keep]<-mapply(post.pred[,keep], post.var[,keep], FUN=function(mu,v){1-normal.logistic(mu,v, approx)})
          post.pred[,keep-length(keep)]<-post.pred[,keep]*exp(post.pred[,keep-length(keep)]+post.var[,keep-length(keep)]/2)
        }
        if(any(grepl("zibinomial", object$Residual$family))){
          keep<-which(object$error.term%in%which(super.trait==k))
          keep<-keep[-c(1:(length(keep)/2))]
          rm.obs<-c(rm.obs, keep)
          post.pred[,keep]<-mapply(post.pred[,keep], post.var[,keep], FUN=function(mu,v){1-normal.logistic(mu,v, approx)})
          post.pred[,keep-length(keep)]<-post.pred[,keep]*mapply(post.pred[,keep-length(keep)], post.var[,keep-length(keep)], FUN=function(mu,v){normal.logistic(mu,v, approx)})
        }
      }
    }
  }
  
  if(length(rm.obs)>0){
    post.pred<-post.pred[,-rm.obs]
  }
  
  if(is.matrix(post.pred)){
    pred<-matrix(colMeans(post.pred), dim(post.pred)[2],1)
  }else{
    pred<-matrix(post.pred, length(post.pred),1)
  }  
  
  if(interval!="none"){
    pred<-cbind(pred, coda::HPDinterval(mcmc(post.pred), prob=level))   
    colnames(pred)<-c("fit", "lwr", "upr")
  }
  
  rownames(pred)<-1:dim(pred)[1]
  
  return(pred)
}
Prediction<-predict.MCMCglmm(mod5, marginal=~leg(mev, -1, FALSE):units)
Precision<-sqrt(1/b$var1)
MR<-b$z1-Prediction
zMR<-MR*Precision
b[,c("zMR","Precision")]<-c(zMR,Precision)

### Egger's regression (on residuals because supposed to be independent)
Egger<-glm(MR~Precision,family="gaussian",data=b)
summary(Egger)
confint(Egger) # CI tells us if there is or not significant publication bias

### Trim and fill method (using residuals too) and test for funnel plot asymmetry
model<-rma.uni(MR,sei=1/Precision,method= "REML") # no bayesian model anymore!
summary(model)
TF1<-trimfill(model,estimator = "R0")
TF2<-trimfill(model,estimator = "L0")
summary(TF1)
summary(TF2)
TF2$beta=as.vector(TF2$beta)
z2=Prediction+MR*TF2$beta # taking estimate for correction from model with L0 (larger nb of missing studies)
prior=list(R=list(V=1,nu=0.002), G=list(G1=list(V=1,nu=0.002),G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))
mod5_corr=MCMCglmm(z2~1,random=~study+parasite_species+animal,mev=var1,prior=prior,pedigree=phylo3,pr=T,nitt=500000,thin=250,burnin=50000,data=b)
summary(mod5_corr)

